#######################################################################
#
# NCM component for authconfig configuration
#
# David Groep <davidg@nikhef.nl>
# Version 1.0.5 : 07/03/05 12:30
#
# ** Generated file : do not edit **
#
#######################################################################

package NCM::Component::authconfig;

use strict;
use NCM::Component;
use LC::Process;
use vars qw(@ISA $EC);
@ISA = qw(NCM::Component);
$EC=LC::Exception::Context->new->will_store_all;
use NCM::Check;

use File::Path;

use EDG::WP4::CCM::Element;


local(*DTA);

##########################################################################
sub getValueDefault() {
##########################################################################
    my ($config, $pathname, $default) = @_;

    if ($config->elementExists($pathname)) {
        return $config->getValue($pathname);
    } else {
      return $default;
    }
}


##########################################################################
sub build_pam_systemauth($$@) {
##########################################################################

    my ($self, $config, $base) = @_;

    # in the /etc/pam.d/system_auth file, some additional stuff can be 
    # set to allow fine-grained access control

    my $conf = &getValueDefault($config,$base."/conffile","/etc/pam.d/system_auth");
    my $changes=0;

    LC::File::copy($conf,$conf."$$",preserve => 1 );
    
    my $section = $config->getValue($base."/section");

    my $method_elmt=$config->getElement("$base/lines");

    while ( $method_elmt->hasNextElement() ) {
        my $m_elmt=$method_elmt->getNextElement();

        my $m_name=$m_elmt->getName();
        my $m_path=($m_elmt->getPath())->toString();
        my $m_value=$config->getValue($m_path."/entry");
        my $m_order=$config->getValue($m_path."/order");
        (my $rem_value=$m_value)=~s/\//\\\//g;
        $rem_value=~s/\$/\\\$/g;
        $rem_value=~s/\s+/\\s+/g;
        # WARNING: this will result in each .so module only to be used once
        # and argument changes to be considered insignificant to functionality
        (my $linere_value=$rem_value)=~s/\.so.*/.so/g;


        my $newline=sprintf("%-12s%s",$section,$m_value);

        $changes+= NCM::Check::lines($conf,
            linere => "^#?\\s*$section\\s+$linere_value\\s*",
            goodre => "^$section\\s+${rem_value}\\s*",
            good   => "$newline",
            keep   => $m_order,
            add    => $m_order
            );
    }

    if ( $changes ) {
        unlink($conf.".old");
        LC::File::move($conf."$$",$conf.".old");
    } else {
        unlink($conf."$$");
    }

    $self->info("Modified $conf in $changes places");
}


##########################################################################
sub build_ldap_config($$@) {
##########################################################################

    my ($self, $config, $base) = @_;

    # from the ldap.conf file, some values are set correctly
    # by the authconfig command. These attributes are
    # "host", "base", "ssl", and "pam_password crypt"

    my $conf = &getValueDefault($config,$base."/conffile","/etc/ldap.conf");
    my $changes=0;

    LC::File::copy($conf,$conf."$$",preserve => 1 );
    
    if ( $config->elementExists($base."/binddn") ) {
        my $binddn=$config->getValue($base."/binddn");
        $changes+= NCM::Check::lines($conf,
            linere => "^#?\\s*binddn\\s+.*",
            goodre => "^binddn\\s+$binddn\\s*",
            good   => "binddn $binddn",
            keep   => "first",
            add    => "last"
            );
    }

    if ( $config->elementExists($base."/bindpw") ) {
        my $bindpw=$config->getValue($base."/bindpw");
        $changes+= NCM::Check::lines($conf,
            linere => "^#?\\s*bindpw\\s+.*",
            goodre => "^bindpw\\s+$bindpw\\s*",
            good   => "bindpw $bindpw",
            keep   => "first",
            add    => "last"
            );
    }

    if ( $config->elementExists($base."/rootbinddn") ) {
        my $rootbinddn=$config->getValue($base."/rootbinddn");
        $changes+= NCM::Check::lines($conf,
            linere => "^#?\\s*rootbinddn\\s+.*",
            goodre => "^rootbinddn\\s+$rootbinddn\\s*",
            good   => "rootbinddn $rootbinddn",
            keep   => "first",
            add    => "last"
            );
    }

    if ( $config->elementExists($base."/port") ) {
        my $port=$config->getValue($base."/port");
        $changes+= NCM::Check::lines($conf,
            linere => "^#?\\s*port\\s+.*",
            goodre => "^port\\s+$port\\s*",
            good   => "port $port",
            keep   => "first",
            add    => "last"
            );
    }

    if ( $config->elementExists($base."/timeouts/idle") ) {
        my $timeo=$config->getValue($base."/timeouts/idle");
        $changes+= NCM::Check::lines($conf,
            linere => "^#?\\s*idle_timelimit\\s+.*",
            goodre => "^idle_timelimit\\s+$timeo\\s*",
            good   => "idle_timelimit $timeo",
            keep   => "first",
            add    => "last"
            );
    }

    if ( $config->elementExists($base."/timeouts/bind") ) {
        my $timeo=$config->getValue($base."/timeouts/bind");
        $changes+= NCM::Check::lines($conf,
            linere => "^#?\\s*bind_timelimit\\s+.*",
            goodre => "^bind_timelimit\\s+$timeo\\s*",
            good   => "bind_timelimit $timeo",
            keep   => "first",
            add    => "last"
            );
    }

    if ( $config->elementExists($base."/timeouts/search") ) {
        my $timeo=$config->getValue($base."/timeouts/search");
        $changes+= NCM::Check::lines($conf,
            linere => "^#?\\s*timelimit\\s+.*",
            goodre => "^timelimit\\s+$timeo\\s*",
            good   => "timelimit $timeo",
            keep   => "first",
            add    => "last"
            );
    }

    if ( $changes ) {
        unlink($conf.".old");
        LC::File::move($conf."$$",$conf.".old");
    } else {
        unlink($conf."$$");
    }

    $self->info("Modified $conf in $changes places");
}

##########################################################################
sub build_authconfig_command($$@) {
##########################################################################

    my ($self, $config, $base) = @_;

    my $cmd="authconfig --kickstart";

    if ( &getValueDefault($config,$base."/usecache","true") eq "true" ) {
      $cmd.=" --enableshadow";
    } else {
      $cmd.=" --disableshadow";
    }

    if ( &getValueDefault($config,$base."/usemd5","false") eq "true" ) {
      $cmd.=" --enablemd5";
    } else {
      $cmd.=" --disablemd5";
    }

    if ( &getValueDefault($config,$base."/usecache","false") eq "true" ) {
      $cmd.=" --enablecache";
    } else {
      $cmd.=" --disablecache";
    }


    # loop over all methods in the configuration and try to
    # recognise them

    my $method_elmt=$config->getElement("$base/method");

    while ( $method_elmt->hasNextElement() ) {
        my $m_elmt=$method_elmt->getNextElement();

        my $m_name=$m_elmt->getName();
        my $m_path=($m_elmt->getPath())->toString();

        if ( $config->getValue($m_path."/enable") ne "true" ) {
            $self->debug(1,"authentication method $m_name set to disabled");
            foreach ( $m_name ) {
            /afs/   and $cmd.=" --disableafs";
            /ldap/  and $cmd.=" --disableldap --disableldapauth";
            /krb5/  and $cmd.=" --disablekrb5";
            /hesiod/ and $cmd.=" --disablehesiod";
            /smb/   and $cmd.=" --disablesmbauth";
            /nis/   and $cmd.=" --disablenis";
            /files/ and $self->warn("Cannot disable file-based auth");
            }
            next;
        }

        # these methods are now to be enabled

        foreach ( $m_name ) {
        /files/ and do {
        };
        /afs/ and do {
            my $afscell=$config->getValue($m_path."/cell");
            $cmd.=" --enableafs --afscell $afscell";
        };
        /nis/ and do {
            my $domain=$config->getValue($m_path."/domain");
            my $servers="";
            my $nissrv_elmt=$config->getElement($m_path."/servers");
            while ( $nissrv_elmt->hasNextElement() ) {
                $servers and $servers.=",";
                $servers.=($nissrv_elmt->getNextElement())->getValue();
            }
            $cmd.=" --enablenis --nisdomain $domain --nisserver $servers";
        };
        /krb5/ and do {
            my $realm=$config->getValue($m_path."/realm");
            my $kdcs="";
            my $krbsrv_elmt=$config->getElement($m_path."/kdcs");
            while ( $krbsrv_elmt->hasNextElement() ) {
                $kdcs and $kdcs.=",";
                $kdcs.=($krbsrv_elmt->getNextElement())->getValue();
            }
            my $adminservers="";
            $krbsrv_elmt=$config->getElement($m_path."/kdcs");
            while ( $krbsrv_elmt->hasNextElement() ) {
                $adminservers and $adminservers.=",";
                $adminservers.=($krbsrv_elmt->getNextElement())->getValue();
            }

            $cmd.=" --enablekrb5 --krb5realm $realm --krb5kdc $kdcs";
            $cmd.=" --krb5adminserver $adminservers";
        };
        /smb/ and do {
            my $wg=$config->getValue($m_path."/workgroup");
            my $servers="";
            my $srv_elmt=$config->getElement($m_path."/servers");
            while ( $srv_elmt->hasNextElement() ) {
                $servers and $servers.=",";
                $servers.=($srv_elmt->getNextElement())->getValue();
            }
            $cmd.=" --enablesmb --smbworkgroup $wg --smbservers $servers";
        };
        /hesiod/ and do {
            my $rhs=$config->getValue($m_path."/rhs");
            my $lhs=$config->getValue($m_path."/lhs");
            $cmd.=" --enablehesiod --hesiodlhs $lhs --hesiodrhs $rhs";
        };
        /ldap/ and do {
            my $nssonly = "false";
            if ( $config->elementExists($m_path."/nssonly")) {
                $nssonly = $config->getValue($m_path."/nssonly");
            }
            if ( $nssonly eq "false" ) { 
                $cmd.=" --enableldapauth --enableldap";
            } else {
                $cmd.=" --disableldapauth --enableldap";
            }

            my $servers="";
            my $srv_elmt=$config->getElement($m_path."/servers");
            while ( $srv_elmt->hasNextElement() ) {
                $servers and $servers.=" ";
                $servers.=($srv_elmt->getNextElement())->getValue();
            }
            $cmd.=" --ldapserver \"$servers\"";

            $cmd.=" --ldapbasedn ".$config->getValue($m_path."/basedn");

            my $usetls=&getValueDefault($config,$m_path."/tls/enable","false");
            if ( $usetls eq "true" ) {
                $cmd.=" --enableldaptls";
            }
        };

        } # foreach ( $m_name )

    } # while ( method )

    return $cmd;
}



##########################################################################
sub Configure($$@) {
##########################################################################
    
    my ($self, $config) = @_;

    # Define paths for convenience. 
    my $base = "/software/components/authconfig";

    my $safemode=&getValueDefault($config,$base."/safemode","false");

    # authconfig basic configuration
    my $authconfig_cmd=$self->build_authconfig_command($config,$base);

    $self->debug(1,"Executing authconfig command with safemode=".$safemode);
    $self->debug(2,"Command = $authconfig_cmd");

    if ( $safemode eq "false" ) { # execute the authconfig command 

        my ($stdout,$stderr);

        my $execute_status = LC::Process::execute( 
                [ $authconfig_cmd ],
                timeout => 60,
                stdout => \$stdout,
                stderr => \$stderr
            );

        if ( $? ) {
            $self->error("authconfig command failed: $?\n$authconfig_cmd");
        }

        if ( $stdout ) {
            $self->info("authconfig command output produced:");
            $self->report($stdout);
        }
        if ( $stderr ) {
            $self->info("authconfig command ERROR produced:");
            $self->report($stderr);
        }

    }

    # the LDAP method has additional configuration in /etc/ldap.conf
    if ( &getValueDefault($config,
            $base."/method/ldap/enable","false") eq "true" ) {

        $self->build_ldap_config($config,$base."/method/ldap");

    }
    if ( $config->elementExists($base."/pamadditions") ) {
      my $method_elmt=$config->getElement("$base/pamadditions");
      while ( $method_elmt->hasNextElement() ) {
        my $m_elmt=$method_elmt->getNextElement();
        my $m_path=($m_elmt->getPath())->toString();
        $self->build_pam_systemauth($config,$m_path);
      }
    }


    return 1;
}

1;      # Required for PERL modules

