############################################################
#
# type definition pro_declaration_component_authconfig
#
# RESPONSIBLE: David Groep <davidg@nikhef.nl>
#
# VERSION : 1.0.5
#
############################################################

declaration template pro_declaration_component_authconfig;

include pro_declaration_component_type;

define type authconfig_pamadditions_line_type = {
  "order"       : string with match(self,"first|last")
  "entry"       : string
};

define type authconfig_pamadditions_type = {
  "conffile"	: string
  "section"     : string with match(self,"auth|account|password|session")
  "lines"       : authconfig_pamadditions_line_type[]
};

define type authconfig_method_generic_type = {
  "enable"	: boolean
};

define type authconfig_method_afs_type = {
  include authconfig_method_generic_type
  "cell"	: string with valid_domain(self)
};

define type authconfig_method_ldap_tls_type = {
  "enable"	: boolean
  "peercheck"	? boolean
  "cacertfile"	? string
  "cacertdir"	? string
  "ciphers"	? string
};

define type authconfig_method_ldap_timeouts_type = {
  "idle"	? long
  "bind"	? long
  "search"	? long
};

define type authconfig_method_ldap_type = {
  include authconfig_method_generic_type
  "servers"	: string[] with valid_domain_list(self)
  "nssonly"	? boolean
  "conffile"	? string
  "basedn"	: string
  "tls"		? authconfig_method_ldap_tls_type
  "binddn"	? string
  "bindpw"	? string
  "rootbinddn"	? string
  "port"	? long with ( self > 0 && self <= 65535 )
  "timeouts"	? authconfig_method_ldap_timeouts_type
};

define type authconfig_method_nis_type = {
  include authconfig_method_generic_type
  "servers"	: string[] with valid_domain_list(self)
  "domain"	: string with valid_domain(self)
};

define type authconfig_method_krb5_type = {
  include authconfig_method_generic_type
  "kdcs"	: string[] with valid_domain_list(self)
  "adminserver"	: string[] with valid_domain_list(self)
  "realm"	: string
};

define type authconfig_method_smb_type = {
  include authconfig_method_generic_type
  "servers"     : string[] with valid_domain_list(self)
  "workgroup"	: string
};

define type authconfig_method_hesiod_type = {
  include authconfig_method_generic_type
  "lhs"		: string
  "rhs"		: string
};

define type authconfig_method_files_type = {
  include authconfig_method_generic_type
};


define type authconfig_method_type = {
  "files"	? authconfig_method_files_type 
  "ldap"	? authconfig_method_ldap_type
  "nis"		? authconfig_method_nis_type
  "krb5"	? authconfig_method_krb5_type
  "smb"		? authconfig_method_smb_type
  "hesiod"	? authconfig_method_hesiod_type
  "afs"		? authconfig_method_afs_type
};

define type component_authconfig_type = {
  include component_type
  "safemode"	? boolean
  "usemd5"	? boolean
  "useshadow"	? boolean
  "usecache"	? boolean
  "startstop"	? boolean

  "method"	? authconfig_method_type

  "pamadditions" ? authconfig_pamadditions_type{}
};

type "/software/components/authconfig" = component_authconfig_type;

